local AddName, AddonTable = ...
local L = AddonTable.L


----------------------------
-- Local Helper Functions --
----------------------------
function AddonTable.druidFormChk()
	if (classFilename == "DRUID") then
		local index = GetShapeshiftForm()
		if (index == 1) or (index == 3) then
			return true
		else
			return false
		end
	end
end

-- Target Armor
local function TargetArmor(self)

	AddonTable.targetArmor = 0
	local NPCguid, NPCname = UnitGUID("target"), UnitName("target")	
	if NPCguid ~= nil then
		local NPCtype, _, _, _, _, npc_id, _ = strsplit("-",NPCguid)
		npc_id = tonumber(npc_id)
			if NPCtype == "Creature" then
				if AddonTable.BossArmor[npc_id] then
					AddonTable.targetArmor = AddonTable.BossArmor[npc_id][1]
				end
			end	
	end
end

-- Stats computing
function AddonTable.StatsCompute()

	-- Init
	AddonTable.buffCount, AddonTable.heroicPresence, AddonTable.zoneBuff, AddonTable.maxmana = 0, 0, 0, 0
	AddonTable.castHaste, AddonTable.meleeHaste, AddonTable.rangedHaste = 100, 100, 100
	AddonTable.arcanePower, AddonTable.combustionCount, AddonTable.evoc, AddonTable.unholyHaste, AddonTable.blessingSanc, AddonTable.rightFury = 0, 0, 0, 0, 0, 0
	AddonTable.masterImp, AddonTable.masterFel, AddonTable.masterSucc, AddonTable.dsSuc, AddonTable.dsImp, AddonTable.felEnergy = 0, 0, 0, 0, 0, 0
	AddonTable.innervate, AddonTable.isShapeshift, AddonTable.treeofLife = 0, false, 0
	AddonTable.aspectViper, AddonTable.icyTalons, AddonTable.vengStacks, AddonTable.vengBuff, AddonTable.avenWrath, AddonTable.epiphany, AddonTable.desolation = 0,0,0,0,0,0,0
	AddonTable.shadowFormDmg, AddonTable.innerFocus, AddonTable.eleMasteryHaste, AddonTable.totemWrath, AddonTable.shadowWeaving, AddonTable.divineProt = 0, 0, 0, 0, 0, 0

	if (AddonTable.classFilename ~= "WARRIOR") and (AddonTable.classFilename ~= "ROGUE") and (AddonTable.classFilename ~= "DEATHKNIGHT") then 

	for i = 1, 40 do
		local _, _, count, _, _, _, _, _, _, spellId = UnitBuff("player",i, "HELPFUL")
		if not spellId  then break end
		
		if spellId then
			AddonTable.buffCount = i
		end
		
		AddonTable.maxmana = UnitPowerMax("player", Enum.PowerType.Mana)
		AddonTable.buffCount = AddonTable.buffCount
		
		if AddonTable.HeroicPresence[spellId] then
			AddonTable.heroicPresence = AddonTable.HeroicPresence[spellId][1]
		end	

		if AddonTable.TotemOfWrath[spellId] then
			AddonTable.totemWrath = AddonTable.TotemOfWrath[spellId][1]
		end

		if AddonTable.InnervateEffect[spellId] then
			AddonTable.innervate = (AddonTable.maxmana * AddonTable.InnervateEffect[spellId][1]) / 10
		end	

		if AddonTable.OutlandsBuffs[spellId] then
			AddonTable.zoneBuff = AddonTable.OutlandsBuffs[spellId][1]
		end
		
		if AddonTable.BlessingOfSanc[spellId] then
			AddonTable.blessingSanc = AddonTable.BlessingOfSanc[spellId][1]
		end

		if AddonTable.Desolation[debuffId] then
			AddonTable.desolation = AddonTable.Desolation[debuffId][1]
		end				
		
	-- Mage
	if (AddonTable.classFilename == "MAGE") then

		AddonTable.maxmana = UnitPowerMax("player", Enum.PowerType.Mana)

		if AddonTable.Combustion[spellId] then
			AddonTable.combustionCount = count * (AddonTable.Combustion[spellId][1])
		end
		
		if AddonTable.IcyVeins[spellId] then
			AddonTable.castHaste = AddonTable.castHaste + (AddonTable.IcyVeins[spellId][1])
		end		

		if AddonTable.Combustion[spellId] then
			AddonTable.combustionCount = count * (AddonTable.Combustion[spellId][1])
		end		

		if AddonTable.Evocation[spellId] then
			AddonTable.evoc = AddonTable.maxmana * (AddonTable.Evocation[spellId][1] / 8) * 2
		end			
		
	end -- end Mage
	
	-- Warlock
	if (AddonTable.classFilename == "WARLOCK") then
	
		local maxpower = UnitPowerMax("player" , mana)
		
		if AddonTable.FelEnergy[spellId] then
			AddonTable.felEnergy = (maxpower * AddonTable.FelEnergy[spellId][1])
		end
		
		if AddonTable.TouchOfShadow[spellId] then
			AddonTable.dsSuc = AddonTable.TouchOfShadow[spellId][1]
		end

		if AddonTable.BurningWish[spellId] then
			AddonTable.dsImp = AddonTable.BurningWish[spellId][1]
		end

		if AddonTable.MasterDemoImp[spellId] then
			AddonTable.masterImp = AddonTable.MasterDemoImp[spellId][1]
		end

		if AddonTable.MasterDemoSucc[spellId] then
			AddonTable.masterSucc = AddonTable.MasterDemoSucc[spellId][1]
		end

		if AddonTable.MasterDemoFel[spellId] then
			AddonTable.masterFel = AddonTable.MasterDemoFel[spellId][1]
		end		
		
		
	end -- end Warlock
		
	if (AddonTable.classFilename == "DRUID") then
	
		if AddonTable.TreeOfLife[spellId] then
			AddonTable.treeofLife = AddonTable.TreeOfLife[spellId][1]
		end		
	
		if GetShapeshiftForm == 1 or GetShapeshiftForm == 3 then
			AddonTable.isShapeshift = true
		else
			AddonTable.isShapeshift = false
		end	
		
	end -- end Druid
	
	-- Hunter
	if (AddonTable.classFilename == "HUNTER") then
	
		local playerMaxMana = UnitPowerMax("player", Enum.PowerType.Mana)	

		if AddonTable.AspectOfViper[spellId] then
			AddonTable.aspectViper = playerMaxMana * AddonTable.AspectOfViper[spellId][1]
		end

	end -- end Hunter
	
	-- Paladin
	if (AddonTable.classFilename == "PALADIN") then
		
		if AddonTable.AvengingWrath[spellId] then
			AddonTable.avenWrath = AddonTable.AvengingWrath[spellId][1]
		end

		if AddonTable.DivineProtection[spellId] then
			AddonTable.divineProt = AddonTable.DivineProtection[spellId][1]
		end

		if (AddonTable.riFuryTalent > 0) and AddonTable.RighteousFury[spellId] then
			AddonTable.rightFury = AddonTable.riFuryTalent
		end
		
		if AddonTable.Vengeance[spellId] then
			AddonTable.vengStacks = count
			if AddonTable.vengeance == 3 and AddonTable.vengStacks > 0 then
				AddonTable.vengBuff = (AddonTable.vengStacks * AddonTable.vengeance) / 100
			elseif AddonTable.vengeance == 2 and AddonTable.vengStacks > 0 then
				AddonTable.vengBuff = (AddonTable.vengStacks * AddonTable.vengeance) / 100					
			elseif AddonTable.vengeance == 1 and AddonTable.vengStacks > 0 then
				AddonTable.vengBuff = (AddonTable.vengStacks * AddonTable.vengeance) / 100				
			else
				AddonTable.vengBuff = 0				
			end
		end			
			
		
	end -- end Paladin
	
	-- Priest
	if (AddonTable.classFilename == "PRIEST") then

		if AddonTable.InnerFocus[spellId] then
			AddonTable.innerFocus = AddonTable.InnerFocus[spellId][1]
		end

		if AddonTable.ShadowForm[spellId] then
			AddonTable.shadowFormDmg = AddonTable.ShadowForm[spellId][1]
		end

		if AddonTable.Epiphany[spellId] then
			AddonTable.epiphany = AddonTable.Epiphany[spellId][1]
		end			
		
		if spellId == 15258 then
			AddonTable.shadowWeaving = count
			AddonTable.shadowWeaving = (AddonTable.shadowWeaving * 2) / 100
		end
		

	end	-- end Paladin

	-- Shaman
	if (AddonTable.classFilename == "SHAMAN") then
		
		if AddonTable.ElementalMastery[spellId] then
			AddonTable.eleMasteryHaste = AddonTable.ElementalMastery[spellId][1]
		end				
		
	end -- end Shaman
	
end -- end Classfilename
end -- end for

	-- Warrior
	if (AddonTable.classFilename == "WARRIOR") then
	
	for i = 1, 40 do
		local _, _, _, _, _, _, _, _, _, spellId = UnitBuff("player",i, "HELPFUL")
		if not spellId then break end   

		if spellId then
			AddonTable.buffCount = i + 1
		end

		if AddonTable.Flurry[spellId] then
			AddonTable.meleeHaste = AddonTable.meleeHaste + AddonTable.Flurry[spellId][1]
		end

		if AddonTable.HeroicPresence[spellId] then
			AddonTable.heroicPresence = AddonTable.HeroicPresence[spellId][1]
		end	

		if AddonTable.TotemOfWrath[spellId] then
			AddonTable.totemWrath = AddonTable.TotemOfWrath[spellId][1]
		end
		
	end -- end Warrior
	
	-- Rogue
	elseif (AddonTable.classFilename == "ROGUE") then
	
	for i = 1, 40 do
		local _, _, _, _, _, _, _, _, _, spellId = UnitBuff("player",i, "HELPFUL")
		if not spellId then break end  

		if spellId then
			AddonTable.buffCount = i
		end
		
		AddonTable.buffCount = AddonTable.buffCount
		
		if AddonTable.SliceAndDice[spellId] then
			AddonTable.meleeHaste = AddonTable.meleeHaste + AddonTable.SliceAndDice[spellId][1]
		end	

		if AddonTable.TotemOfWrath[spellId] then
			AddonTable.totemWrath = AddonTable.TotemOfWrath[spellId][1]
		end			
		
	end -- end Rogue
	
	-- Death Knight
	elseif (AddonTable.classFilename == "DEATHKNIGHT") then
		
		for i = 1, 40 do
			local _, _, _, _, _, _, _, _, _, spellId = UnitBuff("player",i, "HELPFUL")
			if not spellId then break end  

		if spellId then
			AddonTable.buffCount = i
		end			
	
		AddonTable.buffCount = AddonTable.buffCount

		if AddonTable.SliceAndDice[spellId] then
			AddonTable.meleeHaste = AddonTable.meleeHaste + AddonTable.SliceAndDice[spellId][1]
		end	

		if AddonTable.TotemOfWrath[spellId] then
			AddonTable.totemWrath = AddonTable.TotemOfWrath[spellId][1]
		end	

		if AddonTable.IcyTalons[spellId] then
			AddonTable.icyTalons = AddonTable.icyTalons + AddonTable.IcyTalons[spellId][1]
		end
		
		if AddonTable.IcyTalonsImp[spellId] then
			AddonTable.icyTalons = AddonTable.IcyTalonsImp[spellId][1]
		end

		if AddonTable.UnholyPresence[spellId] then
			AddonTable.unholyHaste = AddonTable.UnholyPresence[spellId][1]
		end
		
		if AddonTable.Desolation[debuffId] then
			AddonTable.desolation = AddonTable.Desolation[debuffId][1]
		end			

		end -- end Death Knight
	
	end	 -- end classFilename
	
	AddonTable.hasteEnchants = 0

	local link = GetInventoryItemLink("player", 10)
	if link then
		local itemId, enchantId = link:match("item:(%d+):(%d+)")	
		if enchantId then
			enchantId = tonumber(enchantId)
			if enchantId == 931 then
				AddonTable.hasteEnchants = AddonTable.hasteEnchants + 1
			end
		end
	end
	
	local slotId = GetInventorySlotInfo("HeadSlot")
	local link = GetInventoryItemLink("player", slotId)
	if link then
		local itemId, enchantId = link:match("item:(%d+):(%d+)")
		if enchantId then
			enchantId = tonumber(enchantId)
			if enchantId == 2543 then
				AddonTable.hasteEnchants = AddonTable.hasteEnchants + 1
			end
		end
	end	
	
	local link = GetInventoryItemLink("player", 7)
	if link then
		local itemId, enchantId = link:match("item:(%d+):(%d+)")
		if enchantId then
			enchantId = tonumber(enchantId)
			if enchantId == 2543 then
				AddonTable.hasteEnchants = AddonTable.hasteEnchants + 1
			end
		end
	end	  	
	
end 

--------------------------------------
--		Global OnEvent function		--
--------------------------------------
function AddonTable.OnEventFunc(event, ...)

	if event == "CHARACTER_POINTS_CHANGED" then
	-----	
		AddonTable.talentScan()
		AddonTable.StatsCompute()
	-----	
	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
	-----	
	local event, subevent, _, _, _, _, _, destGUID, _, _, _, spellId, spellName = CombatLogGetCurrentEventInfo()
	local storedGUID = UnitGUID("target")
	local playerGUID = UnitGUID("player")
	AddonTable.manaTide = 0
	
	local manaTideCheck = {
		[16190] = true,
		[39609] = true,
		[39610] = true,
	}
	
	if (manaTideCheck[spellId]) and (destGUID == playerGUID) then
		local TideMana = UnitPowerMax("player", Enum.PowerType.Mana)
		AddonTable.manaTide = ((TideMana * 0.06) / 3) * 2
		AddonTable.manaTide = AddonTable.manaTide
	end
	-----	
	elseif event == "PLAYER_LEVEL_UP" then
	-----	
		AddonTable.playerLevel = UnitLevel("player")
	-----	
	elseif event == "PLAYER_TARGET_CHANGED" then
	-----	
		local exists = UnitExists("target")
		local hostile = UnitCanAttack("player", "target")
	
		if hostile or not exists then
			AddonTable.SinLive()
		end
	
		TargetArmor()
	-----	
	else
	-----	
	
	-- Target checker
	local unit = ...
	if unit == "target" then
		AddonTable.SinLive()
	return
	end
	
------------------------------
--		Stats Updates		--
------------------------------
	AddonTable.StatsCompute()
	
end -- end events
end -- end function